@@ -5,6 +5,7 @@
 #include "./lib/assert.h"
 
 #include "command.h"
+#include "id.h"
 #include "leader.h"
 #include "tracing.h"
 #include "vfs.h"
@@ -13,6 +14,7 @@
  * be invoked. */
 static void leaderExecDone(struct exec *req)
 {
+	tracef("leader exec done id:%" PRIu64, req->id);
 	req->leader->exec = NULL;
 	if (req->cb != NULL) {
 		req->cb(req, req->status);
@@ -25,7 +27,7 @@ static int openConnection(const char *filename,
 			  unsigned page_size,
 			  sqlite3 **conn)
 {
-        tracef("open connection filename %s", filename);
+	tracef("open connection filename %s", filename);
 	char pragma[255];
 	int flags = SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE;
 	char *msg = NULL;
@@ -33,65 +35,77 @@ static int openConnection(const char *filename,
 
 	rc = sqlite3_open_v2(filename, conn, flags, vfs);
 	if (rc != SQLITE_OK) {
-                tracef("open failed %d", rc);
+		tracef("open failed %d", rc);
 		goto err;
 	}
 
 	/* Enable extended result codes */
 	rc = sqlite3_extended_result_codes(*conn, 1);
 	if (rc != SQLITE_OK) {
-                tracef("extended codes failed %d", rc);
-		goto err_after_open;
+		tracef("extended codes failed %d", rc);
+		goto err;
 	}
 
+	/* The vfs, db, gateway, and leader code currently assumes that
+	 * each connection will operate on only one DB file/WAL file
+	 * pair. Make sure that the client can't use ATTACH DATABASE to
+	 * break this assumption. We apply the same limit in open_follower_conn
+	 * in db.c.
+	 *
+	 * Note, 0 instead of 1 -- apparently the "initial database" is not
+	 * counted when evaluating this limit. */
+	sqlite3_limit(*conn, SQLITE_LIMIT_ATTACHED, 0);
+
 	/* Set the page size. */
 	sprintf(pragma, "PRAGMA page_size=%d", page_size);
 	rc = sqlite3_exec(*conn, pragma, NULL, NULL, &msg);
 	if (rc != SQLITE_OK) {
-                tracef("page size set failed %d page size %u", rc, page_size);
-		goto err_after_open;
+		tracef("page size set failed %d page size %u", rc, page_size);
+		goto err;
 	}
 
 	/* Disable syncs. */
 	rc = sqlite3_exec(*conn, "PRAGMA synchronous=OFF", NULL, NULL, &msg);
 	if (rc != SQLITE_OK) {
-                tracef("sync off failed %d", rc);
-		goto err_after_open;
+		tracef("sync off failed %d", rc);
+		goto err;
 	}
 
 	/* Set WAL journaling. */
 	rc = sqlite3_exec(*conn, "PRAGMA journal_mode=WAL", NULL, NULL, &msg);
 	if (rc != SQLITE_OK) {
-                tracef("wal on failed %d", rc);
-		goto err_after_open;
+		tracef("wal on failed %d", rc);
+		goto err;
 	}
 
 	rc = sqlite3_exec(*conn, "PRAGMA wal_autocheckpoint=0", NULL, NULL,
 			  &msg);
 	if (rc != SQLITE_OK) {
-                tracef("wal autocheckpoint off failed %d", rc);
-		goto err_after_open;
+		tracef("wal autocheckpoint off failed %d", rc);
+		goto err;
 	}
 
 	rc =
 	    sqlite3_db_config(*conn, SQLITE_DBCONFIG_NO_CKPT_ON_CLOSE, 1, NULL);
 	if (rc != SQLITE_OK) {
-                tracef("db config failed %d", rc);
-		goto err_after_open;
+		tracef("db config failed %d", rc);
+		goto err;
 	}
 
 	/* TODO: make setting foreign keys optional. */
 	rc = sqlite3_exec(*conn, "PRAGMA foreign_keys=1", NULL, NULL, &msg);
 	if (rc != SQLITE_OK) {
-                tracef("enable foreign keys failed %d", rc);
-		goto err_after_open;
+		tracef("enable foreign keys failed %d", rc);
+		goto err;
 	}
 
 	return 0;
 
-err_after_open:
-	sqlite3_close(*conn);
 err:
+	if (*conn != NULL) {
+		sqlite3_close(*conn);
+		*conn = NULL;
+	}
 	if (msg != NULL) {
 		sqlite3_free(msg);
 	}
@@ -109,14 +123,14 @@ static bool needsBarrier(struct leader *l)
 
 int leader__init(struct leader *l, struct db *db, struct raft *raft)
 {
-        tracef("leader init");
+	tracef("leader init");
 	int rc;
 	l->db = db;
 	l->raft = raft;
-	rc = openConnection(db->filename, db->config->name,
-			    db->config->page_size, &l->conn);
+	rc = openConnection(db->path, db->config->name, db->config->page_size,
+			    &l->conn);
 	if (rc != 0) {
-                tracef("open failed %d", rc);
+		tracef("open failed %d", rc);
 		return rc;
 	}
 
@@ -128,7 +142,7 @@ int leader__init(struct leader *l, struct db *db, struct raft *raft)
 
 void leader__close(struct leader *l)
 {
-        tracef("leader close");
+	tracef("leader close");
 	int rc;
 	/* TODO: there shouldn't be any ongoing exec request. */
 	if (l->exec != NULL) {
@@ -155,15 +169,15 @@ static void leaderCheckpointApplyCb(struct raft_apply *req,
 	(void)result;
 	raft_free(req);
 	if (status != 0) {
-                tracef("checkpoint apply failed %d", status);
+		tracef("checkpoint apply failed %d", status);
 	}
 }
 
 /* Attempt to perform a checkpoint on nodes running a version of dqlite that
  * doens't perform autonomous checkpoints. For recent nodes, the checkpoint
  * command will just be a no-op.
- * This function will run after the WAL might have been checkpointed during a call
- * to `apply_frames`.
+ * This function will run after the WAL might have been checkpointed during a
+ * call to `apply_frames`.
  * */
 static void leaderMaybeCheckpointLegacy(struct leader *l)
 {
@@ -221,7 +235,7 @@ static void leaderApplyFramesCb(struct raft_apply *req,
 				int status,
 				void *result)
 {
-        tracef("apply frames cb");
+	tracef("apply frames cb id:%" PRIu64, idExtract(req->req_id));
 	struct apply *apply = req->data;
 	struct leader *l = apply->leader;
 	if (l == NULL) {
@@ -232,7 +246,7 @@ static void leaderApplyFramesCb(struct raft_apply *req,
 	(void)result;
 
 	if (status != 0) {
-                tracef("apply frames cb failed status %d", status);
+		tracef("apply frames cb failed status %d", status);
 		sqlite3_vfs *vfs = sqlite3_vfs_find(l->db->config->name);
 		switch (status) {
 			case RAFT_LEADERSHIPLOST:
@@ -259,7 +273,7 @@ static void leaderApplyFramesCb(struct raft_apply *req,
 				l->exec->status = SQLITE_IOERR;
 				break;
 		}
-		VfsAbort(vfs, l->db->filename);
+		VfsAbort(vfs, l->db->path);
 	}
 
 	raft_free(apply);
@@ -278,6 +292,7 @@ static int leaderApplyFrames(struct exec *req,
 			     dqlite_vfs_frame *frames,
 			     unsigned n)
 {
+	tracef("leader apply frames id:%" PRIu64, req->id);
 	struct leader *l = req->leader;
 	struct db *db = l->db;
 	struct command_frames c;
@@ -295,24 +310,25 @@ static int leaderApplyFrames(struct exec *req,
 
 	apply = raft_malloc(sizeof *req);
 	if (apply == NULL) {
-                tracef("malloc");
+		tracef("malloc");
 		rv = DQLITE_NOMEM;
 		goto err;
 	}
 
 	rv = command__encode(COMMAND_FRAMES, &c, &buf);
 	if (rv != 0) {
-                tracef("encode %d", rv);
+		tracef("encode %d", rv);
 		goto err_after_apply_alloc;
 	}
 
 	apply->leader = req->leader;
 	apply->req.data = apply;
 	apply->type = COMMAND_FRAMES;
+	idSet(apply->req.req_id, req->id);
 
 	rv = raft_apply(l->raft, &apply->req, &buf, 1, leaderApplyFramesCb);
 	if (rv != 0) {
-                tracef("raft apply failed %d", rv);
+		tracef("raft apply failed %d", rv);
 		goto err_after_command_encode;
 	}
 
@@ -332,37 +348,51 @@ err:
 
 static void leaderExecV2(struct exec *req)
 {
+	tracef("leader exec v2 id:%" PRIu64, req->id);
 	struct leader *l = req->leader;
 	struct db *db = l->db;
 	sqlite3_vfs *vfs = sqlite3_vfs_find(db->config->name);
 	dqlite_vfs_frame *frames;
+	uint64_t size;
 	unsigned n;
 	unsigned i;
 	int rv;
 
 	req->status = sqlite3_step(req->stmt);
 
-	rv = VfsPoll(vfs, l->db->filename, &frames, &n);
+	rv = VfsPoll(vfs, db->path, &frames, &n);
 	if (rv != 0 || n == 0) {
-                tracef("vfs poll");
+		tracef("vfs poll");
 		goto finish;
 	}
 
+	/* Check if the new frames would create an overfull database */
+	size = VfsDatabaseSize(vfs, db->path, n, db->config->page_size);
+	if (size > VfsDatabaseSizeLimit(vfs)) {
+		rv = SQLITE_FULL;
+		goto abort;
+	}
+
 	rv = leaderApplyFrames(req, frames, n);
+	if (rv != 0) {
+		goto abort;
+	}
+
 	for (i = 0; i < n; i++) {
 		sqlite3_free(frames[i].data);
 	}
 	sqlite3_free(frames);
-	if (rv != 0) {
-		VfsAbort(vfs, l->db->filename);
-		goto finish;
-	}
-
 	return;
 
+abort:
+	for (i = 0; i < n; i++) {
+		sqlite3_free(frames[i].data);
+	}
+	sqlite3_free(frames);
+	VfsAbort(vfs, l->db->path);
 finish:
 	if (rv != 0) {
-                tracef("exec v2 failed %d", rv);
+		tracef("exec v2 failed %d", rv);
 		l->exec->status = rv;
 	}
 	leaderExecDone(l->exec);
@@ -370,7 +400,7 @@ finish:
 
 static void execBarrierCb(struct barrier *barrier, int status)
 {
-        tracef("exec barrier cb status %d", status);
+	tracef("exec barrier cb status %d", status);
 	struct exec *req = barrier->data;
 	struct leader *l = req->leader;
 	if (status != 0) {
@@ -384,24 +414,27 @@ static void execBarrierCb(struct barrier *barrier, int status)
 int leader__exec(struct leader *l,
 		 struct exec *req,
 		 sqlite3_stmt *stmt,
+		 uint64_t id,
 		 exec_cb cb)
 {
-        tracef("leader exec");
+	tracef("leader exec id:%" PRIu64, id);
 	int rv;
 	if (l->exec != NULL) {
-                tracef("busy");
+		tracef("busy");
 		return SQLITE_BUSY;
 	}
 	l->exec = req;
 
 	req->leader = l;
 	req->stmt = stmt;
+	req->id = id;
 	req->cb = cb;
 	req->barrier.data = req;
 	req->barrier.cb = NULL;
 
 	rv = leader__barrier(l, &req->barrier, execBarrierCb);
 	if (rv != 0) {
+		l->exec = NULL;
 		return rv;
 	}
 	return 0;
@@ -409,7 +442,7 @@ int leader__exec(struct leader *l,
 
 static void raftBarrierCb(struct raft_barrier *req, int status)
 {
-        tracef("raft barrier cb status %d", status);
+	tracef("raft barrier cb status %d", status);
 	struct barrier *barrier = req->data;
 	int rv = 0;
 	if (status != 0) {
@@ -430,10 +463,10 @@ static void raftBarrierCb(struct raft_barrier *req, int status)
 
 int leader__barrier(struct leader *l, struct barrier *barrier, barrier_cb cb)
 {
-        tracef("leader barrier");
+	tracef("leader barrier");
 	int rv;
 	if (!needsBarrier(l)) {
-                tracef("not needed");
+		tracef("not needed");
 		cb(barrier, 0);
 		return 0;
 	}
@@ -442,7 +475,7 @@ int leader__barrier(struct leader *l, struct barrier *barrier, barrier_cb cb)
 	barrier->req.data = barrier;
 	rv = raft_barrier(l->raft, &barrier->req, raftBarrierCb);
 	if (rv != 0) {
-                tracef("raft barrier failed %d", rv);
+		tracef("raft barrier failed %d", rv);
 		barrier->req.data = NULL;
 		barrier->leader = NULL;
 		barrier->cb = NULL;
