@@ -1,8 +1,6 @@
 #ifndef DQLITE_SERVER_H
 #define DQLITE_SERVER_H
 
-#include <raft.h>
-#include <raft/uv.h>
 #include <sqlite3.h>
 
 #include <semaphore.h>
@@ -12,6 +10,7 @@
 #include "id.h"
 #include "lib/assert.h"
 #include "logger.h"
+#include "raft.h"
 #include "registry.h"
 
 #define DQLITE_ERRMSG_BUF_SIZE 300
@@ -43,9 +42,8 @@ struct dqlite_node
 	struct uv_async_s handover;
 	int handover_status;
 	void (*handover_done_cb)(struct dqlite_node *, int);
-	struct uv_async_s stop;      /* Trigger UV loop stop */
-	struct uv_timer_s startup;   /* Unblock ready sem */
-	struct uv_prepare_s monitor; /* Raft state change monitor */
+	struct uv_async_s stop;    /* Trigger UV loop stop */
+	struct uv_timer_s startup; /* Unblock ready sem */
 	struct uv_timer_s timer;
 	int raft_state;     /* Previous raft state */
 	char *bind_address; /* Listen address */
